# -*- coding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2009 Nexedi SARL and Contributors. All Rights Reserved.
# Copyright (c) 2006-2008 Zope Corporation and Contributors.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
import os, shutil, sys
import zc.buildout
import plone.recipe.zope2instance

class Recipe(plone.recipe.zope2instance.Recipe):
    def install(self):
        # Override plone.recipe.zope2instance so as to create several
        # directories used by ERP5.
        options = self.options
        location = options['location']

        requirements, ws = self.egg.working_set()
        ws_locations = [d.location for d in ws]

        if os.path.exists(location):
            # patch begin: call self._clean_up instead of shutil.rmtree
            self._clean_up(location)
            # patch end: call self._clean_up instead of shutil.rmtree

        # What follows is a bit of a hack because the instance-setup mechanism
        # is a bit monolithic. We'll run mkzopeinstance and then we'll
        # patch the result. A better approach might be to provide independent
        # instance-creation logic, but this raises lots of issues that
        # need to be stored out first.
        if not self.zope2_location:
            mkzopeinstance = os.path.join(
                options['bin-directory'], 'mkzopeinstance')
            if not mkzopeinstance:
                # EEE
                return

        else:
            mkzopeinstance = os.path.join(
                self.zope2_location, 'bin', 'mkzopeinstance.py')
            if not os.path.exists(mkzopeinstance):
                mkzopeinstance = os.path.join(
                    self.zope2_location, 'utilities', 'mkzopeinstance.py')
            if sys.platform[:3].lower() == "win":
                mkzopeinstance = '"%s"' % mkzopeinstance

        if not mkzopeinstance:
            # EEE
            return

        assert os.spawnl(
            os.P_WAIT, os.path.normpath(options['executable']),
            zc.buildout.easy_install._safe_arg(options['executable']),
            mkzopeinstance, '-d',
            zc.buildout.easy_install._safe_arg(location),
            '-u', options['user'],
            ) == 0

        # patch begin: create several directories and a symlink for ERP5
        for directory in ('Constraint', 'Document', 'PropertySheet', 'tests'):
            path = os.path.join(location, directory)
            if not os.path.exists(path):
                os.mkdir(path)
        for directory in ('bt5',):
            path = os.path.join(location, directory)
            if os.path.exists(path) and not os.path.islink(path):
                shutil.rmtree(path)
            if not os.path.exists(path):
                os.symlink(os.path.join('..', directory), path)
        # patch end: create several directories and a symlink for ERP5

        try:
            # Save the working set:
            open(os.path.join(location, 'etc', '.eggs'), 'w').write(
                '\n'.join(ws_locations))

            # Make a new zope.conf based on options in buildout.cfg
            self.build_zope_conf()

            # Patch extra paths into binaries
            self.patch_binaries(ws_locations)

            # Install extra scripts
            self.install_scripts()

            # Add zcml files to package-includes
            self.build_package_includes()
        except:
            # clean up
            if os.path.exists(location):
                # patch begin: call self._clean_up instead of shutil.rmtree
                self._clean_up(location)
                # patch end: call self._clean_up instead of shutil.rmtree
            raise

        return location

    def _clean_up(self, location):
        # do not delete contents in several directories that can be
        # modified by business template installation.
        for path in os.listdir(location):
            if path in ('Constraint', 'Document', 'Extensions',
                        'PropertySheet', 'tests',):
                pass
            else:
                path = os.path.join(location, path)
                if os.path.isdir(path) and not os.path.islink(path):
                    shutil.rmtree(path)
                else:
                    os.unlink(path)
